@@ -10,6 +10,9 @@ CONV_KERNEL_INITIALIZER = keras.initializers.VarianceScaling(scale=2.0, mode="fa
 # CONV_KERNEL_INITIALIZER = 'glorot_uniform'
 
 
+""" Wrapper for default parameters """
+
+
 @tf.keras.utils.register_keras_serializable(package="kecamCommon")
 def hard_swish(inputs):
     """ `out = xx * relu6(xx + 3) / 6`, arxiv: https://arxiv.org/abs/1905.02244 """
@@ -57,8 +60,131 @@ def activation_by_name(inputs, activation="relu", name=None):
         return keras.layers.Activation(activation=activation, name=layer_name)(inputs)
 
 
-def batchnorm_with_activation(inputs, activation="relu", zero_gamma=False, epsilon=BATCH_NORM_EPSILON, momentum=BATCH_NORM_DECAY, act_first=False, name=None):
+@tf.keras.utils.register_keras_serializable(package="kecamCommon")
+class EvoNormalization(tf.keras.layers.Layer):
+    def __init__(self, nonlinearity=True, num_groups=-1, zero_gamma=False, momentum=0.99, epsilon=0.001, data_format="auto", **kwargs):
+        # [evonorm](https://github.com/tensorflow/tpu/blob/master/models/official/resnet/resnet_model.py)
+        # EVONORM_B0: nonlinearity=True, num_groups=-1
+        # EVONORM_S0: nonlinearity=True, num_groups > 0
+        # EVONORM_B0 / EVONORM_S0 linearity: nonlinearity=False, num_groups=-1
+        # EVONORM_S0A linearity: nonlinearity=False, num_groups > 0
+        super().__init__(**kwargs)
+        self.data_format, self.nonlinearity, self.zero_gamma, self.num_groups = data_format, nonlinearity, zero_gamma, num_groups
+        self.momentum, self.epsilon = momentum, epsilon
+        self.is_channels_first = True if data_format == "channels_first" or (data_format == "auto" and K.image_data_format() == "channels_first") else False
+
+    def build(self, input_shape):
+        all_axes = list(range(len(input_shape)))
+        param_shape = [1] * len(input_shape)
+        if self.is_channels_first:
+            param_shape[1] = input_shape[1]
+            self.reduction_axes = all_axes[:1] + all_axes[2:]
+        else:
+            param_shape[-1] = input_shape[-1]
+            self.reduction_axes = all_axes[:-1]
+
+        self.gamma = self.add_weight(name="gamma", shape=param_shape, initializer="zeros" if self.zero_gamma else "ones", trainable=True)
+        self.beta = self.add_weight(name="beta", shape=param_shape, initializer="zeros", trainable=True)
+        if self.num_groups <= 0:  # EVONORM_B0
+            self.moving_variance = self.add_weight(
+                name="moving_variance",
+                shape=param_shape,
+                initializer="ones",
+                synchronization=tf.VariableSynchronization.ON_READ,
+                trainable=False,
+                aggregation=tf.VariableAggregation.MEAN,
+            )
+        if self.nonlinearity:
+            self.vv = self.add_weight(name="vv", shape=param_shape, initializer="ones", trainable=True)
+
+        if self.num_groups > 0:  # EVONORM_S0
+            channels_dim = input_shape[1] if self.is_channels_first else input_shape[-1]
+            num_groups = int(self.num_groups)
+            while num_groups > 1:
+                if channels_dim % num_groups == 0:
+                    break
+                num_groups -= 1
+            self.__num_groups__ = num_groups
+            self.groups_dim = channels_dim // self.__num_groups__
+
+            if self.is_channels_first:
+                self.group_shape = [-1, self.__num_groups__, self.groups_dim, *input_shape[2:]]
+                self.group_reduction_axes = list(range(2, len(self.group_shape)))  # [2, 3, 4]
+                self.group_axes = 2
+                self.var_shape = [-1, *param_shape[1:]]
+            else:
+                self.group_shape = [-1, *input_shape[1:-1], self.__num_groups__, self.groups_dim]
+                self.group_reduction_axes = list(range(1, len(self.group_shape) - 2)) + [len(self.group_shape) - 1]  # [1, 2, 4]
+                self.group_axes = -1
+                self.var_shape = [-1, *param_shape[1:]]
+
+    def __group_std__(self, inputs):
+        # _group_std, https://github.com/tensorflow/tpu/blob/main/models/official/resnet/resnet_model.py#L171
+        grouped = tf.reshape(inputs, self.group_shape)
+        _, var = tf.nn.moments(grouped, self.group_reduction_axes, keepdims=True)
+        std = tf.sqrt(var + self.epsilon)
+        std = tf.repeat(std, self.groups_dim, axis=self.group_axes)
+        return tf.reshape(std, self.var_shape)
+
+    def __batch_std__(self, inputs, training=None):
+        # _batch_std, https://github.com/tensorflow/tpu/blob/main/models/official/resnet/resnet_model.py#L120
+        def _call_train_():
+            _, var = tf.nn.moments(inputs, self.reduction_axes, keepdims=True)
+            # update_op = tf.assign_sub(moving_variance, (moving_variance - variance) * (1 - decay))
+            delta = (self.moving_variance - var) * (1 - self.momentum)
+            self.moving_variance.assign_sub(delta)
+            return var
+
+        def _call_test_():
+            return self.moving_variance
+
+        var = K.in_train_phase(_call_train_, _call_test_, training=training)
+        return tf.sqrt(var + self.epsilon)
+
+    def __instance_std__(self, inputs):
+        # _instance_std, https://github.com/tensorflow/tpu/blob/main/models/official/resnet/resnet_model.py#L111
+        # axes = [1, 2] if data_format == 'channels_last' else [2, 3]
+        _, var = tf.nn.moments(inputs, self.reduction_axes[1:], keepdims=True)
+        return tf.sqrt(var + self.epsilon)
+
+    def call(self, inputs, training=None, **kwargs):
+        if self.nonlinearity and self.num_groups > 0:  # EVONORM_S0
+            den = self.__group_std__(inputs)
+            inputs = inputs * tf.nn.sigmoid(self.vv * inputs) / den
+        elif self.num_groups > 0:  # EVONORM_S0a
+            # EvoNorm2dS0a https://github.com/rwightman/pytorch-image-models/blob/main/timm/models/layers/evo_norm.py#L239
+            den = self.__group_std__(inputs)
+            inputs = inputs / den
+        elif self.nonlinearity:  # EVONORM_B0
+            left = self.__batch_std__(inputs, training)
+            right = self.vv * inputs + self.__instance_std__(inputs)
+            inputs = inputs / tf.maximum(left, right)
+        return inputs * self.gamma + self.beta
+
+    def get_config(self):
+        config = super().get_config()
+        config.update(
+            {
+                "nonlinearity": self.nonlinearity,
+                "zero_gamma": self.zero_gamma,
+                "num_groups": self.num_groups,
+                "momentum": self.momentum,
+                "epsilon": self.epsilon,
+                "data_format": self.data_format,
+            }
+        )
+        return config
+
+
+def batchnorm_with_activation(
+    inputs, activation=None, zero_gamma=False, epsilon=1e-5, momentum=0.9, act_first=False, use_evo_norm=False, evo_norm_group_size=-1, name=None
+):
     """ Performs a batch normalization followed by an activation. """
+    if use_evo_norm:
+        nonlinearity = False if activation is None else True
+        num_groups = inputs.shape[-1] // evo_norm_group_size  # Currently using gorup_size as parameter only
+        return EvoNormalization(nonlinearity, num_groups=num_groups, zero_gamma=zero_gamma, epsilon=epsilon, momentum=momentum, name=name + "evo_norm")(inputs)
+
     bn_axis = -1 if K.image_data_format() == "channels_last" else 1
     gamma_initializer = tf.zeros_initializer() if zero_gamma else tf.ones_initializer()
     if act_first and activation:
@@ -127,35 +253,7 @@ def depthwise_conv2d_no_bias(inputs, kernel_size, strides=1, padding="VALID", us
     )(inputs)
 
 
-def deep_stem(inputs, stem_width, activation="relu", last_strides=1, name=None):
-    nn = conv2d_no_bias(inputs, stem_width // 2, 3, strides=2, padding="same", name=name and name + "1_")
-    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "1_")
-    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "2_")
-    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "2_")
-    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "3_")
-    return nn
-
-
-def quad_stem(inputs, stem_width, activation="relu", stem_act=False, last_strides=2, name=None):
-    nn = conv2d_no_bias(inputs, stem_width // 8, 3, strides=2, padding="same", name=name and name + "1_")
-    if stem_act:
-        nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "1_")
-    nn = conv2d_no_bias(nn, stem_width // 4, 3, strides=1, padding="same", name=name and name + "2_")
-    if stem_act:
-        nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "2_")
-    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "3_")
-    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "3_")
-    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "4_")
-    return nn
-
-
-def tiered_stem(inputs, stem_width, activation="relu", last_strides=1, name=None):
-    nn = conv2d_no_bias(inputs, 3 * stem_width // 8, 3, strides=2, padding="same", name=name and name + "1_")
-    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "1_")
-    nn = conv2d_no_bias(nn, stem_width // 2, 3, strides=1, padding="same", name=name and name + "2_")
-    nn = batchnorm_with_activation(nn, activation=activation, name=name and name + "2_")
-    nn = conv2d_no_bias(nn, stem_width, 3, strides=last_strides, padding="same", name=name and name + "3_")
-    return nn
+""" Blocks """
 
 
 def output_block(inputs, filters=0, activation="relu", num_classes=1000, drop_rate=0, classifier_activation="softmax", is_torch_mode=True, act_first=False):
@@ -224,6 +322,9 @@ def drop_block(inputs, drop_rate=0, name=None):
         return inputs
 
 
+""" Other layers / functions """
+
+
 @tf.keras.utils.register_keras_serializable(package="kecamCommon")
 def __anti_alias_downsample_initializer__(weight_shape, dtype="float32"):
     import numpy as np
